package com.yfd.monitor.vmanager.gdw2019.record;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.yfd.monitor.common.StreamInfo;
import com.yfd.monitor.common.VideoManagerConstants;
import com.yfd.monitor.conf.exception.ControllerException;
import com.yfd.monitor.conf.exception.SsrcTransactionNotFoundException;
import com.yfd.monitor.gdw2019.bean.Device;
import com.yfd.monitor.gdw2019.bean.DeviceChannel;
import com.yfd.monitor.gdw2019.bean.RecordInfo;
import com.yfd.monitor.gdw2019.transmit.callback.DeferredResultHolder;
import com.yfd.monitor.gdw2019.transmit.callback.RequestMessage;
import com.yfd.monitor.gdw2019.transmit.cmd.impl.SIPCommander;
import com.yfd.monitor.service.IDeviceService;
import com.yfd.monitor.service.IPlayService;
import com.yfd.monitor.storager.IVideoManagerStorage;
import com.yfd.monitor.utils.DateUtil;
import com.yfd.monitor.vmanager.bean.DeferredResultEx;
import com.yfd.monitor.vmanager.bean.ErrorCode;
import com.yfd.monitor.vmanager.bean.StreamContent;
import com.yfd.monitor.vmanager.bean.WVPResult;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.scheduling.annotation.Async;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.context.request.async.DeferredResult;

import javax.sip.InvalidArgumentException;
import javax.sip.SipException;
import java.text.ParseException;
import java.util.Collection;
import java.util.UUID;

@Tag(name  = "国标录像")

@RestController
@RequestMapping("/api/gb_record")
public class GBRecordController {

	private final static Logger logger = LoggerFactory.getLogger(GBRecordController.class);

	@Autowired
	private SIPCommander cmder;

	@Autowired
	private IVideoManagerStorage storager;

	@Autowired
	private DeferredResultHolder resultHolder;

	@Autowired
	private IPlayService playService;

	@Autowired
	private IDeviceService deviceService;

    @Autowired
    private RedisTemplate<Object, Object> redisTemplate;


	@Operation(summary = "硬盘录像机视频录像查询")
	@Parameter(name = "deviceId", description = "设备国标编号", required = true)
	@Parameter(name = "channelId", description = "通道国标编号", required = true)
	@Parameter(name = "startTime", description = "开始时间", required = true)
	@Parameter(name = "endTime", description = "结束时间", required = true)
	@GetMapping("/queryNvrRecord/{deviceId}/{channelId}")
	public DeferredResult<WVPResult<RecordInfo>> queryNvrRecord(@PathVariable String deviceId, @PathVariable String channelId, String startTime, String endTime) throws InterruptedException {

		if (logger.isDebugEnabled()) {
			logger.debug(String.format("录像信息查询 API调用，deviceId：%s ，startTime：%s， endTime：%s", deviceId, startTime,
					endTime));
		}
		DeferredResult<WVPResult<RecordInfo>> result = new DeferredResult<>();
		if (!DateUtil.verification(startTime, DateUtil.formatter)) {
			throw new ControllerException(ErrorCode.ERROR100.getCode(), "startTime格式为" + DateUtil.PATTERN);
		}
		if (!DateUtil.verification(endTime, DateUtil.formatter)) {
			throw new ControllerException(ErrorCode.ERROR100.getCode(), "endTime格式为" + DateUtil.PATTERN);
		}
		//根据摄像头id,查询出对应Nvr设备
		Device device = storager.queryNvrDeviceByDeviceId(deviceId, channelId);
		// 指定超时时间 1分钟30秒
		String uuid = UUID.randomUUID().toString();
		int sn = (int) ((Math.random() * 9 + 1) * 100000);
		String key = DeferredResultHolder.CALLBACK_CMD_RECORDINFO + device.getDeviceId() + sn;
		RequestMessage msg = new RequestMessage();
		msg.setId(uuid);
		msg.setKey(key);
		try {
			// customName 为硬盘录像机对应的摄像机通道
			cmder.recordInfoQuery(device, device.getCustomName(), startTime, endTime, sn, null, null, null,
					(eventResult -> {
				WVPResult<RecordInfo> wvpResult = new WVPResult<>();
				wvpResult.setCode(ErrorCode.ERROR100.getCode());
				wvpResult.setMsg("查询录像失败, status: " +  eventResult.statusCode + ", message: " + eventResult.msg);
				msg.setData(wvpResult);
				resultHolder.invokeResult(msg);
			}));
		} catch (InvalidArgumentException | SipException | ParseException e) {
			logger.error("[命令发送失败] 查询录像: {}", e.getMessage());
			throw new ControllerException(ErrorCode.ERROR100.getCode(), "命令发送失败: " + e.getMessage());
		}

		// 录像查询以channelId作为deviceId查询
		resultHolder.put(key, uuid, result);
		result.onTimeout(() -> {
			msg.setData("timeout");
			WVPResult<RecordInfo> wvpResult = new WVPResult<>();
			wvpResult.setCode(ErrorCode.ERROR100.getCode());
			wvpResult.setMsg("timeout");
			msg.setData(wvpResult);
			resultHolder.invokeResult(msg);
		});
		return result;
	}

    @Operation(summary = "查询摄像机录像时长")
    @Parameter(name = "deviceId", description = "设备国标编号", required = true)
    @Parameter(name = "channelId", description = "通道国标编号", required = true)
    @GetMapping("/queryRecordTimes/{deviceId}/{channelId}")
    public String queryRecordTimes(@PathVariable String deviceId, @PathVariable String channelId) throws InterruptedException {
        String RecordTimeKey = VideoManagerConstants.REDIS_RECORD_TIMES_PRE +channelId ;
        String recordtimes=redisTemplate.opsForValue().get(RecordTimeKey).toString();
        return recordtimes;
    }


	@Operation(summary = "录像查询")
	@Parameter(name = "deviceId", description = "设备国标编号", required = true)
	@Parameter(name = "channelId", description = "通道国标编号", required = true)
	@Parameter(name = "startTime", description = "开始时间", required = true)
	@Parameter(name = "endTime", description = "结束时间", required = true)
	@GetMapping("/query/{deviceId}/{channelId}")
	public DeferredResult<WVPResult<RecordInfo>> recordinfo(@PathVariable String deviceId,
															@PathVariable String channelId, String startTime,
															String endTime) {

		if (logger.isDebugEnabled()) {
			logger.debug(String.format("录像信息查询 API调用，deviceId：%s ，startTime：%s， endTime：%s", deviceId, startTime,
					endTime));
		}
		DeferredResult<WVPResult<RecordInfo>> result = new DeferredResult<>();
		if (!DateUtil.verification(startTime, DateUtil.formatter)) {
			throw new ControllerException(ErrorCode.ERROR100.getCode(), "startTime格式为" + DateUtil.PATTERN);
		}
		if (!DateUtil.verification(endTime, DateUtil.formatter)) {
			throw new ControllerException(ErrorCode.ERROR100.getCode(), "endTime格式为" + DateUtil.PATTERN);
		}

		Device device = storager.queryVideoDevice(deviceId);
		// 指定超时时间 1分钟30秒
		String uuid = UUID.randomUUID().toString();
		int sn = (int) ((Math.random() * 9 + 1) * 100000);
		String key = DeferredResultHolder.CALLBACK_CMD_RECORDINFO + deviceId + sn;
		RequestMessage msg = new RequestMessage();
		msg.setId(uuid);
		msg.setKey(key);
		try {
            cmder.recordInfoQuery(device, channelId, startTime, endTime, sn, null, null, null, (eventResult -> {
                WVPResult<RecordInfo> wvpResult = new WVPResult<>();
                wvpResult.setCode(ErrorCode.ERROR100.getCode());
                wvpResult.setMsg("查询录像失败, status: " +  eventResult.statusCode + ", message: " + eventResult.msg);
                msg.setData(wvpResult);
                resultHolder.invokeResult(msg);
            }));
        } catch (InvalidArgumentException | SipException | ParseException e) {
            logger.error("[命令发送失败] 查询录像: {}", e.getMessage());
            throw new ControllerException(ErrorCode.ERROR100.getCode(), "命令发送失败: " +  e.getMessage());
        }

        // 录像查询以channelId作为deviceId查询
        resultHolder.put(key, uuid, result);
        result.onTimeout(()->{
            msg.setData("timeout");
            WVPResult<RecordInfo> wvpResult = new WVPResult<>();
            wvpResult.setCode(ErrorCode.ERROR100.getCode());
            wvpResult.setMsg("timeout");
            msg.setData(wvpResult);
            resultHolder.invokeResult(msg);
        });
        return result;
    }
    @Operation(summary = "硬盘录像机录像下载")
    @Parameter(name = "deviceId", description = "设备国标编号", required = true)
    @Parameter(name = "channelId", description = "通道国标编号", required = true)
    @Parameter(name = "startTime", description = "开始时间", required = true)
    @Parameter(name = "endTime", description = "结束时间", required = true)
    @Parameter(name = "downloadSpeed", description = "下载倍速", required = true)
    @GetMapping("/downloadRecord/start/{deviceId}/{channelId}")
    public DeferredResult<WVPResult<StreamContent>> downloadRecord(@PathVariable String deviceId, @PathVariable String channelId,
                                                             String startTime, String endTime, String downloadSpeed) {

		Device nvrdevice = storager.queryNvrDeviceByDeviceId(deviceId, channelId);
        deviceId=nvrdevice.getDeviceId();//硬盘录像机
        if (logger.isDebugEnabled()) {
            logger.debug(String.format("历史媒体下载 API调用，deviceId：%s，channelId：%s，downloadSpeed：%s", deviceId, channelId, downloadSpeed));
        }

        String uuid = UUID.randomUUID().toString();
        String key = DeferredResultHolder.CALLBACK_CMD_DOWNLOAD + deviceId + channelId;
        DeferredResult<WVPResult<StreamContent>> result = new DeferredResult<>(30000L);
        resultHolder.put(key, uuid, result);
        RequestMessage msg = new RequestMessage();
        msg.setId(uuid);
        msg.setKey(key);

        WVPResult<StreamContent> wvpResult = new WVPResult<>();

        playService.download(deviceId, channelId, startTime, endTime, Integer.parseInt(downloadSpeed), null, playBackResult->{

            wvpResult.setCode(playBackResult.getCode());
            wvpResult.setMsg(playBackResult.getMsg());
            if (playBackResult.getCode() == ErrorCode.SUCCESS.getCode()) {
                StreamInfo streamInfo = (StreamInfo)playBackResult.getData();
                wvpResult.setData(new StreamContent(streamInfo));
            }
            msg.setData(wvpResult);
            resultHolder.invokeResult(msg);
        });

        return result;
    }

	@Operation(summary = "开始历史媒体下载")
	@Parameter(name = "deviceId", description = "设备国标编号", required = true)
	@Parameter(name = "channelId", description = "通道国标编号", required = true)
	@Parameter(name = "startTime", description = "开始时间", required = true)
	@Parameter(name = "endTime", description = "结束时间", required = true)
	@Parameter(name = "downloadSpeed", description = "下载倍速", required = true)
	@GetMapping("/download/start/{deviceId}/{channelId}")
	public DeferredResult<WVPResult<StreamContent>> download(@PathVariable String deviceId, @PathVariable String channelId,
													   String startTime, String endTime, String downloadSpeed) {

		if (logger.isDebugEnabled()) {
			logger.debug(String.format("历史媒体下载 API调用，deviceId：%s，channelId：%s，downloadSpeed：%s", deviceId, channelId, downloadSpeed));
		}

		String uuid = UUID.randomUUID().toString();
		String key = DeferredResultHolder.CALLBACK_CMD_DOWNLOAD + deviceId + channelId;
		DeferredResult<WVPResult<StreamContent>> result = new DeferredResult<>(30000L);
		resultHolder.put(key, uuid, result);
		RequestMessage msg = new RequestMessage();
		msg.setId(uuid);
		msg.setKey(key);

		WVPResult<StreamContent> wvpResult = new WVPResult<>();

		playService.download(deviceId, channelId, startTime, endTime, Integer.parseInt(downloadSpeed), null, playBackResult->{

			wvpResult.setCode(playBackResult.getCode());
			wvpResult.setMsg(playBackResult.getMsg());
			if (playBackResult.getCode() == ErrorCode.SUCCESS.getCode()) {
				StreamInfo streamInfo = (StreamInfo)playBackResult.getData();
				wvpResult.setData(new StreamContent(streamInfo));
			}
			msg.setData(wvpResult);
			resultHolder.invokeResult(msg);
		});

		return result;
	}

	@Operation(summary = "停止历史媒体下载")
	@Parameter(name = "deviceId", description = "设备国标编号", required = true)
	@Parameter(name = "channelId", description = "通道国标编号", required = true)
	@Parameter(name = "stream", description = "流ID", required = true)
	@GetMapping("/download/stop/{deviceId}/{channelId}/{stream}")
	public void playStop(@PathVariable String deviceId, @PathVariable String channelId, @PathVariable String stream) {

		if (logger.isDebugEnabled()) {
			logger.debug(String.format("设备历史媒体下载停止 API调用，deviceId/channelId：%s_%s", deviceId, channelId));
		}

		if (deviceId == null || channelId == null) {
			throw new ControllerException(ErrorCode.ERROR400);
		}

		Device device = deviceService.getDevice(deviceId);
		if (device == null) {
			throw new ControllerException(ErrorCode.ERROR400.getCode(), "设备：" + deviceId + "未找到");
		}

		try {
			cmder.streamByeCmd(device, channelId, stream, null);
		} catch (InvalidArgumentException | ParseException | SipException | SsrcTransactionNotFoundException e) {
			logger.error("[停止历史媒体下载]停止历史媒体下载，发送BYE失败 {}", e.getMessage());
			throw new ControllerException(ErrorCode.ERROR100.getCode(), e.getMessage());
		}
	}

	@Operation(summary = "获取历史媒体下载进度")
	@Parameter(name = "deviceId", description = "设备国标编号", required = true)
	@Parameter(name = "channelId", description = "通道国标编号", required = true)
	@Parameter(name = "stream", description = "流ID", required = true)
	@GetMapping("/download/progress/{deviceId}/{channelId}/{stream}")
	public StreamContent getProgress(@PathVariable String deviceId, @PathVariable String channelId, @PathVariable String stream) {
		StreamInfo downLoadInfo = playService.getDownLoadInfo(deviceId, channelId, stream);
		if (downLoadInfo == null) {
			throw new ControllerException(ErrorCode.ERROR404);
		}
		return new StreamContent(downLoadInfo);
	}
}
