package com.yfd.platform.config;

import lombok.Data;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.stereotype.Component;

import java.util.Map;

/**
 * 腾讯短信业务配置类(sms)
 *
 * 注意：@Component 和 @ConfigurationProperties(prefix = "tencent.sms") 需要同时使用
 * @version 1.0
 * @date 2021/12/30 18:32
 */
@Data
@Component
@ConfigurationProperties(prefix = "tencent.sms")
public class TencentSmsProperties {
    /** 腾讯云账户密钥对secretId */
    private String secretId;

    /** 腾讯云账户密钥对secretKey */
    private String secretKey;

    /** 短信应用ID: 短信SdkAppId在 [短信控制台] 添加应用后生成的实际SdkAppId（位于[应用管理]中的[应用列表]），示例: 1400006666 (1400开头)*/
    private String sdkAppId;

    /** 短信签名内容: 使用 UTF-8 编码，必须填写已审核通过的签名，签名信息可登录 [短信控制台] 查看 */
    private String signName;

    /** 模板 ID 哈希表: 必须填写已审核通过的模板 ID。模板ID可登录 [短信控制台] 查看
     *  key: 模板名称(自定义)；value: 模板 ID（腾讯云已通过模板）
     * */
    private Map<String, String> templateIdMap;

    /** 国际/港澳台短信 SenderId: 国内短信填空，默认未开通(国内短信不需要填写此项)，如需开通请联系 [sms helper] */
    String senderid = "";


    /**短信号码扩展号: 默认未开通，如需开通请联系 [sms helper]   个人不需要填写*/
    private String extendCode = "";

}

