package com.yfd.platform.modules.basedata.domain;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import lombok.Data;
import lombok.EqualsAndHashCode;

import java.io.Serializable;

/**
 * <p>
 * 设备下的通道信息(每个设备查看视频的主要信息表)

 * </p>
 *
 *
 * @since 2023-04-21
 */
@Data
@EqualsAndHashCode(callSuper = false)
@TableName("mon_device_channel")
public class DeviceChannel implements Serializable {

    private static final long serialVersionUID = 1L;

    /**
     * 数据库自增ID
     */
    @TableId(value = "id", type = IdType.AUTO)
    private Integer id;

    /**
     * 通道国标编号
     */
    @TableField("channelId")
    private String channelid;

    /**
     * 设备国标编号
     */
    @TableField("deviceId")
    private String deviceid;

    /**
     * 通道名称
     */
    private String name;

    /**
     * 生产厂商
     */
    private String manufacture;

    /**
     * 号型
     */
    private String model;

    /**
     * 设备归属
     */
    private String owner;

    /**
     * 行政区域
     */
    @TableField("civilCode")
    private String civilcode;

    /**
     * 警区
     */
    private String block;

    /**
     * 安装地址
     */
    private String address;

    /**
     * 是否有子设备 1有, 0没有
     */
    private String parental;

    /**
     * 父级id
     */
    @TableField("parentId")
    private String parentid;

    /**
     * 信令安全模式  缺省为0; 0:不采用; 2: S/MIME签名方式; 3: S/ MIME加密签名同时采用方式; 4:数字摘要方式
     */
    @TableField("safetyWay")
    private Integer safetyway;

    /**
     * 注册方式 缺省为1;1:符合IETFRFC3261标准的认证注册模 式; 2:基于口令的双向认证注册模式; 3:基于数字证书的双向认证注册模式
     */
    @TableField("registerWay")
    private Integer registerway;

    /**
     * 证书序列号
     */
    @TableField("certNum")
    private String certnum;

    /**
     * 证书有效标识 缺省为0;证书有效标识:0:无效1: 有效
     */
    private Integer certifiable;

    /**
     * 证书无效原因码
     */
    @TableField("errCode")
    private Integer errcode;

    /**
     * 证书终止有效期
     */
    @TableField("endTime")
    private String endtime;

    /**
     * 保密属性 缺省为0; 0:不涉密, 1:涉密
     */
    private String secrecy;

    /**
     * IP地址
     */
    @TableField("ipAddress")
    private String ipaddress;

    /**
     * 端口号
     */
    private Integer port;

    /**
     * 密码
     */
    private String password;

    /**
     * 云台类型
     */
    @TableField("PTZType")
    private Integer ptztype;

    /**
     * 创建时间
     */
    @TableField("createTime")
    private String createtime;

    /**
     * 更新时间
     */
    @TableField("updateTime")
    private String updatetime;

    /**
     * 在线/离线， 1在线,0离线
     */
    private Integer status;

    /**
     * 经度
     */
    private Double longitude;

    /**
     * 纬度
     */
    private Double latitude;

    /**
     * GCJ02坐标系经度
     */
    @TableField("longitudeGcj02")
    private Double longitudegcj02;

    /**
     * GCJ02坐标系纬度
     */
    @TableField("latitudeGcj02")
    private Double latitudegcj02;

    /**
     * WGS84坐标系经度
     */
    @TableField("longitudeWgs84")
    private Double longitudewgs84;

    /**
     * WGS84坐标系纬度
     */
    @TableField("latitudeWgs84")
    private Double latitudewgs84;

    /**
     * 子设备数
     */
    @TableField("subCount")
    private Integer subcount;

    /**
     * 流唯一编号，存在表示正在直播
     */
    @TableField("streamId")
    private String streamid;

    /**
     * 是否含有音频
     */
    @TableField("hasAudio")
    private Boolean hasaudio;

    /**
     * 业务分组
     */
    @TableField("businessGroupId")
    private String businessgroupid;

    /**
     * GPS更新时间
     */
    @TableField("gpsTime")
    private String gpstime;


}
